package com.yizhi.live.application.service;


import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.IService;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.vo.DroolsVo;
import com.yizhi.live.application.domain.LiveActivity;
import com.yizhi.live.application.request.LiveReleaseReviewReq;
import com.yizhi.live.application.vo.*;
import com.yizhi.util.application.domain.BizResponse;
import org.apache.xpath.operations.Bool;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 直播活动表 服务类
 * </p>
 *
 * @author fulan123
 * @since 2018-04-10
 */
public interface ILiveActivityService extends IService<LiveActivity> {

    /**
     *  创建直播间
     *
     * @param params
     * @return
     */
    String createChanel(Map<String, String> params);

    Page<PageLiveVo> searchPage(String title, String startTimeString, String endTimeString,
                                Long companyId, Long siteId,
                                List<Long> orgId, int pageNo, int pageSize,
                                String expTimeString, Integer status , Integer viewType,Boolean startTimeAsc,Boolean endTimeAsc);

    /**
     * 查询近七天的已完成的直播
     * @param companyId
     * @param siteId
     * @return
     */
    List<LiveActivity> findWithinSevenDaysLive(Long companyId, Long siteId, Long accountId);

    Boolean deleteChanel(String chanel);

    String setToken(String chanel);


    List<PageLiveVo> apiSearchHomeList(List<Long> relationIds,String title,
                                   Long companyId, Long siteId,
                                   int pageNo, int pageSize);


    Page<PageLiveVo> apiSearchPageList(List<Long> relationIds,String title,
                                   Long companyId, Long siteId,
                                   int pageNo, int pageSize);


    Boolean saveLive(LiveActivityResultVO liveActivityResultVO);

    Boolean updateLive(LiveActivityResultVO liveActivityResultVO);

    List<PageLiveVo> portalList(String title, Date startTime, Date endTime,
                                Long companyId, Long siteId,
                                List<Long> orgId, int pageNo, int pageSize);

    Page<PageLiveVo> portalList2(String title, Date startTime, Date endTime,
                                Long companyId, Long siteId,
                                List<Long> orgId, int pageNo, int pageSize);
    
    
    /**
 	 * 范围人员导出             管理端-------2018/09/19
 	 * @param assignmentId
 	 * @return
 	 */
 	public VisibleRangeExport vsibleRangeExport(Long liveId);

    /**
     * 根据relationId 判断这个人是否可以看到这这场直播
     * @param relationId  里面可能有accountId or orgId
     * @param liveId       直播Id
     * @return
     */
    Boolean judgeScope (List<Long> relationId,Long liveId);

    Page<PageLiveVo> getPageLiveVoList(Long siteId, Integer pageNo, Integer pageSize);
    Page<LiveActivity> getPageLiveVoListNotIds(List<Long> ids, Long siteId, Integer pageNo, Integer pageSize);
    Page<LiveActivity> LiveActivityByIds(List<Long> listIds, Integer pageNo, Integer pageSize);

    /**
     * 根据传入的频道Id,返回直播状态
     * @param channelId
     * @return
     */
    Integer getLiveStatus(String channelId);

    /**
     * 根据传入的relationIds 里面有accountId、orgId
     * @param relationIds
     * @param pageNo
     * @param pageSize
     * @return
     */
    List<PageLiveVo> queryLiveListByRelationIds( List<Long> relationIds, Integer pageNo, Integer pageSize);
    
    
    List<Map<String, Object>> getServerByCompanyIdAndIds(Long companyId,List<Long> ids);

    Page<PageLiveVo> pcSearchPageList(List<Long> relationIds, String title, Long companyId, Long siteId, int pageNo, int pageSize);

    Page<PageLiveVo> getPageToCalendar(Date date, Page<PageLiveVo> page);

    Page<DroolsVo> getPageByDrools(String field, String value, Page<DroolsVo> page);

    /**
     * 获取直播的实时在线人数
     * @param channelIds 多个直播号用逗号隔开
     * @return 在线人数list
     */
    List<LiveOnlineCountVo> getChannelOnlineCountRealTime(String channelIds) throws JsonProcessingException;

    /**
     * 直播状态修改；保利威回调接口
     * @param channelId 频道id
     */
    BizResponse liveStatusCallback(String channelId, String status, Long timestamp, String sign, String sessionId, String startTime, String endTime);

    /**
     * 回放生成成功；保利威回调接口
     */
    BizResponse livebackCallback(PolyvLiveBackCallbackVo liveBackCallbackVo);

    /**
     * 直播转存成功；保利威回调接口
     */
    BizResponse<String> liveTransferCallback(PolyvLiveBackCallbackVo liveBackCallbackVo);

    /**
     * 查询有直播回放的直播列表
     * @param liveReviewsVo 参数
     */
    BizResponse<List<LiveTitleVo>> reviewLiveList(LiveReviewsVo liveReviewsVo);

    /**
     * 查询有直播回放的直播间
     * 分页查询
     * @param replayVo 入参  pageNo;pageSize
     * @return 直播间列表
     */
    Page<LiveTitleVo> getHasReplayLiveList(LiveReleaseReviewReq replayVo);

    /**
     * 查询直播间所有回放视频
     * @param liveReviewsVo 参数
     */
    BizResponse<List<LiveReplayVo>> reviewsList(LiveReviewsVo liveReviewsVo);

    /**
     * 获取开启直播回放的直播间ids
     * @param context 上下文
     * @return 直播间ids
     */
    List<Long> getReplayOnLiveIds(RequestContext context);


    /**
     * 管理端查询直播下的直播回放
     * @param liveReviewsVo liveId 直播id
     */
    Page<LiveReplayVo> getReplayList(LiveReviewsVo liveReviewsVo);

    /**
     * 直播回放上架、下架
     * @param replayId 回放id
     * @param operation true上架；false下架
     */
    Boolean updateReplayStatus(Long replayId, Boolean operation);

    /**
     * 直播回放上架、下架
     * @param replayId 回放id
     * @param top true上架；false下架
     */
    Boolean updateReplayTop(Long replayId, Boolean top);

    /**
     * 修改回放主播名和封面
     * @param infoVo 修改信息
     */
    Boolean updateReplayInfos(LiveReplayInfoVo infoVo);
}
