package com.yizhi.live.application.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.enums.InternationalEnums;
import com.yizhi.live.application.domain.LiveActivity;
import com.yizhi.live.application.domain.LiveReplay;
import com.yizhi.live.application.mapper.LiveActivityMapper;
import com.yizhi.live.application.mapper.LiveReplayMapper;
import com.yizhi.live.application.request.LiveReleaseReviewReq;
import com.yizhi.live.application.service.ILiveActivityService;
import com.yizhi.live.application.service.ILiveReplayService;
import com.yizhi.live.application.util.PolyvUtils;
import com.yizhi.live.application.vo.LiveReplayVo;
import com.yizhi.live.application.vo.LiveReviewListVo;
import com.yizhi.live.application.vo.LiveStatisticDataVo;
import com.yizhi.live.application.vo.PageLiveVo;
import com.yizhi.util.application.domain.BizResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Date 2020/12/9 9:47 上午
 * @Author lvjianhui
 **/
@Service
public class ILiveReplayServiceImpl extends ServiceImpl<LiveReplayMapper, LiveReplay> implements ILiveReplayService {

    private static final Logger logger = LoggerFactory.getLogger(ILiveReplayServiceImpl.class);
    @Autowired
    private ILiveActivityService liveActivityService;
    @Autowired
    private LiveReplayMapper replayMapper;
    @Autowired
    private RedisCache redisCache;
    @Autowired
    private LiveActivityMapper liveActivityMapper;

    @Override
    public void deleteByUpdateTime(Date updateTime) {
        replayMapper.deleteByUpdateTime(updateTime);
    }


    @Override
    public Page<LiveReplayVo> getReplayList(LiveReviewListVo liveReviewListVo) {

        Page<LiveReplayVo> page = new Page<>(liveReviewListVo.getPageNo(),liveReviewListVo.getPageSize());
        /*List<LiveReplayVo> replayVoList = new ArrayList<>();
        Map<String,Integer> liveViewTypeMap = new HashMap<>();
        page.setRecords(replayVoList);
        List<Long> liveIdList = liveReviewListVo.getLiveIdList();
        if (null == liveIdList || liveIdList.isEmpty()){
            liveIdList = new ArrayList<>();
            RequestContext context = liveReviewListVo.getRequestContext();
            Page<PageLiveVo> liveActivityPage = liveActivityService.apiSearchPageList(context.getRelationIds(), null, context.getCompanyId(), context.getSiteId(), 1, Integer.MAX_VALUE);
            if (null == liveActivityPage || null == liveActivityPage.getRecords()){
                logger.warn("查询直播列表为空;accountId = {},companyId = {},siteId ={}",context.getAccountId(),context.getCompanyId(),context.getSiteId());
                return page;
            }
            List<PageLiveVo> records = liveActivityPage.getRecords();
            liveIdList = records.stream().map(PageLiveVo::getId).collect(Collectors.toList());
            // 获取直播类型
            for (PageLiveVo pageLiveVo : records) {
                String channel = pageLiveVo.getChannel();
                Integer viewType = pageLiveVo.getViewType();
                liveViewTypeMap.put(channel,viewType);
            }
        }
        logger.info("直播id listSize= {}", liveIdList.size());
        List<LiveReplayVo> list = replayMapper.selectReplayList(liveIdList,liveReviewListVo.getTitle(),page);
        if (null == list || list.isEmpty()){
            logger.info("获取到直播回放列表size为空");
            return page;
        }
        // 查询直播观看人数
        String hKey = "live:viewer";
        list.forEach(liveReplayVo -> {
           //redis中获取；接口有限制
           String channelId = liveReplayVo.getChannelId();
            Integer viewType = liveViewTypeMap.get(channelId);
            liveReplayVo.setViewType(viewType);
            String viewerSum = (String) redisCache.hget(hKey,channelId);
           if (!StringUtils.isEmpty(viewerSum)){
               liveReplayVo.setLiveViewers(Integer.valueOf(viewerSum));
           }else {
               Date livePlanStartAt = liveReplayVo.getLivePlanStartAt();
               List<LiveStatisticDataVo> liveStatistic = PolyvUtils.getLiveStatistic(channelId, livePlanStartAt, new Date());
               if (null != liveStatistic && liveStatistic.size() > 0){
                   for (LiveStatisticDataVo liveStatisticDataVo : liveStatistic) {
                       Integer mobileUniqueViewer = liveStatisticDataVo.getMobileUniqueViewer();
                       Integer pcUniqueViewer = liveStatisticDataVo.getPcUniqueViewer();
                       Integer sum = mobileUniqueViewer + pcUniqueViewer;
                       liveReplayVo.setLiveViewers(sum);
                       redisCache.hset(hKey,channelId,String.valueOf(sum));
                       //过期时间60s
                       redisCache.expire(hKey,60);
                   }
               }
           }
        });
        page.setRecords(list);*/
        return page;
    }

    @Override
    public BizResponse<LiveReplayVo> getReplay(Long replayId) {

        if (null == replayId){
            logger.error("查询直播回放详情；直播回放id为空!");
            return BizResponse.fail(InternationalEnums.INTERNATIONALIZATIONWORDCONTROLLER1.getCode());
        }
        LiveReplay liveReplay = replayMapper.selectById(replayId);
        LiveReplayVo replayVo = new LiveReplayVo();
        BeanUtils.copyProperties(liveReplay,replayVo);
        Long liveId = replayVo.getLiveId();
        LiveActivity activity = liveActivityMapper.selectById(liveId);
        if (null != activity){
            replayVo.setViewType(activity.getViewType());
        }
        return BizResponse.ok(replayVo);
    }

    @Override
    public LiveReplay getByVideoPoolId(String videoPoolId) {
        LiveReplay lr = new LiveReplay();
                lr.setVideoPoolId(videoPoolId);
        return this.baseMapper.selectOne(lr);
    }


    @Override
    public Page<LiveReplayVo> getReleaseReplayList(LiveReleaseReviewReq reviewReq) {
        // 获取上架的直播间回放
        int pageNo = reviewReq.getPageNo();
        int pageSize = reviewReq.getPageSize();
        if (pageNo == 0 || pageSize == 0){
            pageNo = 1;
            pageSize = Integer.MAX_VALUE;
        }
        Page<LiveReplayVo> page = new Page<>(pageNo,pageSize);
        List<LiveReplayVo> replayVoList = new ArrayList<>();
        Map<String,Integer> liveViewTypeMap = new HashMap<>();
        page.setRecords(replayVoList);
        RequestContext context = reviewReq.getRequestContext();
        List<Long> liveIdList = liveActivityService.getReplayOnLiveIds(context);
        if (null == liveIdList || liveIdList.isEmpty()){
            logger.error("查询直播回放列表，直播间获取为空;companyId = {} , siteId = {} ,accountId = {}",context.getCompanyId(),context.getSiteId(),context.getAccountId());
            return page;
        }
        logger.info("直播id listSize= {}", liveIdList.size());
        List<LiveReplayVo> list = replayMapper.selectReplayList(liveIdList,reviewReq.getTitle(),page);
        if (null == list || list.isEmpty()){
            logger.info("获取到直播回放列表size为空");
            return page;
        }
        // 查询直播观看人数
        String hKey = "live:viewer";
        list.forEach(liveReplayVo -> {
            //redis中获取；接口有限制
            String channelId = liveReplayVo.getChannelId();
            String viewerSum = (String) redisCache.hget(hKey,channelId);
            if (!StringUtils.isEmpty(viewerSum)){
                liveReplayVo.setLiveViewers(Integer.valueOf(viewerSum));
            }else {
                Date livePlanStartAt = liveReplayVo.getLivePlanStartAt();
                List<LiveStatisticDataVo> liveStatistic = PolyvUtils.getLiveStatistic(channelId, livePlanStartAt, new Date());
                if (null != liveStatistic && liveStatistic.size() > 0){
                    for (LiveStatisticDataVo liveStatisticDataVo : liveStatistic) {
                        Integer mobileUniqueViewer = liveStatisticDataVo.getMobileUniqueViewer();
                        Integer pcUniqueViewer = liveStatisticDataVo.getPcUniqueViewer();
                        Integer sum = mobileUniqueViewer + pcUniqueViewer;
                        liveReplayVo.setLiveViewers(sum);
                        redisCache.hset(hKey,channelId,String.valueOf(sum));
                        //过期时间60s
                        redisCache.expire(hKey,60);
                    }
                }
            }
        });
        page.setRecords(list);
        return page;
    }
}
